﻿/*
  BNO055用サブルーチン

  I2Cで接続されたBNO055と通信。

  ・元ネタはこちら
    https://github.com/BoschSensortec/BNO055_driver
    細かい調整の類いをせずひとまず動かすといった点に絞るとこのソースにあ
    る程度で良いのではと
  ・BNO055の設定関数類は戻り値で正否を判定可能だが、内部の処理の途中で何
    かしら失敗すると後処理せずに即失敗を返す
  ・MPUとBNO055間はI2Cのみで接続されており、I/O等でやりとりすべき機能は
    何一つ使用できない
*/

#ifndef INC_E097B_IMU_H
#define INC_E097B_IMU_H

#ifdef __cplusplus
extern "C" {
#endif

#include  <chip.h>
#include  <wl_i2c.h>
#include  <wl_delay.h>

// マクロ・変数など
//-----------------------------
#define _BNO055_SLAVE_ADDR    0x28  // COM3=GND

// Page 0 Registers
#define BNO055_P0ADDR_CHIP_ID           0x00
#define BNO055_P0ADDR_ACC_ID            0x01
#define BNO055_P0ADDR_MAG_ID            0x02
#define BNO055_P0ADDR_GYR_ID            0x03
#define BNO055_P0ADDR_SW_REV_ID         0x04
#define BNO055_P0ADDR_BL_Rev_ID         0x06
#define BNO055_ADDR_Page_ID             0x07
#define BNO055_P0ADDR_ACC_DATA_X        0x08
#define BNO055_P0ADDR_ACC_DATA_Y        0x0A
#define BNO055_P0ADDR_ACC_DATA_Z        0x0C
#define BNO055_P0ADDR_MAG_DATA_X        0x0E
#define BNO055_P0ADDR_MAG_DATA_Y        0x10
#define BNO055_P0ADDR_MAG_DATA_Z        0x12
#define BNO055_P0ADDR_GYR_DATA_X        0x14
#define BNO055_P0ADDR_GYR_DATA_Y        0x16
#define BNO055_P0ADDR_GYR_DATA_Z        0x18
#define BNO055_P0ADDR_EUL_Heading       0x1A
#define BNO055_P0ADDR_EUL_Roll          0x1C
#define BNO055_P0ADDR_EUL_Pitch         0x1E
#define BNO055_P0ADDR_QUA_Data_w        0x20
#define BNO055_P0ADDR_QUA_Data_x        0x22
#define BNO055_P0ADDR_QUA_Data_y        0x24
#define BNO055_P0ADDR_QUA_Data_z        0x26
#define BNO055_P0ADDR_LIA_Data_X        0x28
#define BNO055_P0ADDR_LIA_Data_Y        0x2A
#define BNO055_P0ADDR_LIA_Data_Z        0x2C
#define BNO055_P0ADDR_GRV_Data_X        0x2E
#define BNO055_P0ADDR_GRV_Data_Y        0x30
#define BNO055_P0ADDR_GRV_Data_Z        0x32
#define BNO055_P0ADDR_TEMP              0X34
#define BNO055_P0ADDR_CALIB_STAT        0x35
#define BNO055_P0ADDR_ST_RESULT         0x36
#define BNO055_P0ADDR_INTR_STAT         0x37
#define BNO055_P0ADDR_SYS_CLK_STAT      0x38
#define BNO055_P0ADDR_SYS_STAT          0x39
#define BNO055_P0ADDR_SYS_ERR           0x3A
#define BNO055_P0ADDR_UNIT_SEL          0x3B
#define BNO055_P0ADDR_OPR_MODE          0x3D
#define BNO055_P0ADDR_PWR_MODE          0x3E
#define BNO055_P0ADDR_SYS_TRIGGER       0x3F
#define BNO055_P0ADDR_TEMP_SOURCE       0x40
#define BNO055_P0ADDR_AXIS_MAP_CONFIG   0x41
#define BNO055_P0ADDR_AXIS_MAP_SIGN     0x42
#define BNO055_P0ADDR_ACC_OFFSET_X      0x55
#define BNO055_P0ADDR_ACC_OFFSET_Y      0x57
#define BNO055_P0ADDR_ACC_OFFSET_Z      0x59
#define BNO055_P0ADDR_MAG_OFFSET_X      0x5B
#define BNO055_P0ADDR_MAG_OFFSET_Y      0x5D
#define BNO055_P0ADDR_MAG_OFFSET_Z      0x5F
#define BNO055_P0ADDR_GYR_OFFSET_X      0x61
#define BNO055_P0ADDR_GYR_OFFSET_Y      0x63
#define BNO055_P0ADDR_GYR_OFFSET_Z      0x65
#define BNO055_P0ADDR_ACC_RADIUS        0x67
#define BNO055_P0ADDR_MAG_RADIUS        0x69

// Page 1 Registers
#define BNO055_P1ADDR_ACC_CONFIG        0x08
#define BNO055_P1ADDR_MAG_CONFIG        0x09
#define BNO055_P1ADDR_GYR_CONFIG_0      0x0A
#define BNO055_P1ADDR_GYR_CONFIG_1      0x0B
#define BNO055_P1ADDR_ACC_Sleep_Config  0x0C
#define BNO055_P1ADDR_GYR_Sleep_Config  0x0D
#define BNO055_P1ADDR_INT_MSK           0x0F
#define BNO055_P1ADDR_INT_EN            0x10
#define BNO055_P1ADDR_ACC_AM_THRES      0x11
#define BNO055_P1ADDR_ACC_INT_Settings  0x12
#define BNO055_P1ADDR_ACC_HG_DURATION   0x13
#define BNO055_P1ADDR_ACC_HG_THRES      0x14
#define BNO055_P1ADDR_ACC_NM_THRES      0x15
#define BNO055_P1ADDR_ACC_NM_SET        0x16
#define BNO055_P1ADDR_GYR_INT_SETTING   0x17
#define BNO055_P1ADDR_GYR_HR_X_SET      0x18
#define BNO055_P1ADDR_GYR_DUR_X         0x19
#define BNO055_P1ADDR_GYR_HR_Y_SET      0x1A
#define BNO055_P1ADDR_GYR_DUR_Y         0x1B
#define BNO055_P1ADDR_GYR_HR_Z_SET      0x1C
#define BNO055_P1ADDR_GYR_DUR_Z         0x1D
#define BNO055_P1ADDR_GYR_AM_THRES      0x1E
#define BNO055_P1ADDR_GYR_AM_SET        0x1F
#define BNO055_P1ADDR_UNIQUE_ID         0x50

// Operation Mode
typedef enum {
  opmode_CONFIG, opmode_ACCONLY, opmode_MAGONLY, opmode_GYRONLY, opmode_ACCMAG, opmode_ACCGYRO, opmode_MAGGYRO, opmode_AMG, opmode_IMUPLUS, opmode_COMPASS, opmode_M4G, opmode_NDOF_FMC_OFF, opmode_NDOF
} TOperationMode;

// Power Mode
typedef enum {
  powmode_NORMAL, powmode_LOWPOWER, powmode_SUSPEND
} TPowerMode;

// Axis Placement
typedef enum {
  axisplace_P0, axisplace_P1, axisplace_P2, axisplace_P3, axisplace_P4, axisplace_P5, axisplace_P6, axisplace_P7
} TAxisPlace;

static const uint8_t axisplace_conf_sign[8][2] = {
  { 0x21, 0x04 },
  { 0x24, 0x00 },
  { 0x24, 0x06 },
  { 0x21, 0x02 },
  { 0x24, 0x03 },
  { 0x21, 0x01 },
  { 0x21, 0x07 },
  { 0x24, 0x05 }
};

// Sensor & Fusion data
typedef struct {
  int16_t acc_x;    // Acceleration Data
  int16_t acc_y;
  int16_t acc_z;
  int16_t mag_x;    // Magnetometer Data
  int16_t mag_y;
  int16_t mag_z;
  int16_t gyro_x;   // Gyroscope Data
  int16_t gyro_y;
  int16_t gyro_z;
  int16_t heading;  // Euler Data
  int16_t roll;
  int16_t pitch;
  int16_t q_w;      // Quaternion Data
  int16_t q_x;
  int16_t q_y;
  int16_t q_z;
  int16_t lia_x;    // Linear Acceleration Data
  int16_t lia_y;
  int16_t lia_z;
  int16_t grv_x;    // Gravity Vector Data
  int16_t grv_y;
  int16_t grv_z;
  int8_t  temp;     // Temperature
} __attribute__ ((packed)) Tbno055;

// I2Cのパラメータ (データ幅が8ビットの場合はライブラリ側でエンディアン処理を行わない)
static const TIICDataSpec ds = { IIC_Reg8, IIC_Data8, IIC_LittleEndian };

// サブルーチン
//-----------------------------
// Page切り替え
static bool imu_change_page (uint8_t page) {
  static uint8_t prevpage = 99;
  if (prevpage == page) return true;

  uint8_t a = BNO055_ADDR_Page_ID, d = page;
  bool result = (IIC_Write (LPC_I2C0, _BNO055_SLAVE_ADDR, &a, &d, 1, &ds, NULL));
//  bool result = imu_write_byte (BNO055_ADDR_Page_ID, page & 1);
  if (result) prevpage = page;
  return result;
}

// 指定レジスタアドレスへ1バイト書き込み
bool imu_write_byte (uint8_t page, uint8_t addr, uint8_t dat) {
  uint8_t a = addr, d = dat;
  if (!imu_change_page (page)) return false;
  return (IIC_Write (LPC_I2C0, _BNO055_SLAVE_ADDR, &a, &d, 1, &ds, NULL));
}

// 指定レジスタアドレスから1バイト読み出し
bool imu_read_byte (uint8_t page, uint8_t addr, uint8_t *dat) {
  uint8_t a = addr;
  if (!imu_change_page (page)) return false;
  return IIC_Read (LPC_I2C0, _BNO055_SLAVE_ADDR, &a, dat, 1, &ds, NULL);
}

// 測定データ及び計算値を一括読み出し
bool imu_read_datavalue (Tbno055 *r) {
  uint8_t regaddr = BNO055_P0ADDR_ACC_DATA_X;
  if (!imu_change_page (0)) return false;
  return IIC_Read (LPC_I2C0, _BNO055_SLAVE_ADDR, &regaddr, r, sizeof (Tbno055), &ds, NULL);
}

// CHIP IDの確認 (主に疎通確認用)
bool imu_cehckid (void) {
  uint8_t dat;
  for (int i = 0; i < 100; i++) {
    if (imu_read_byte (0, BNO055_P0ADDR_CHIP_ID, &dat)) return (dat == 0xa0);
    Delay_ms (10);
  }
  return false;
}

// Operation Modeの取得
bool imu_get_opmode (TOperationMode *mode) {
  return imu_read_byte (0, BNO055_P0ADDR_OPR_MODE, mode);
}

// Operation Modeの設定
bool imu_set_opmode (TOperationMode mode, TOperationMode *pprevmode) {
  TOperationMode prevmode;
  if (imu_get_opmode (&prevmode)) {
    if (pprevmode != NULL) *pprevmode = prevmode;
    if (mode == prevmode) return true;
    else {
      // CONFIG -> other
      if (prevmode == opmode_CONFIG) {
        if (!imu_write_byte (0, BNO055_P0ADDR_OPR_MODE, mode)) return false;
        Delay_ms (600);
        return true;
        // other -> other or CONFIG
      } else {
        if (!imu_write_byte (0, BNO055_P0ADDR_OPR_MODE, opmode_CONFIG)) return false;
        Delay_ms (20);
        if (mode != opmode_CONFIG) {
          if (!imu_write_byte (0, BNO055_P0ADDR_OPR_MODE, mode)) return false;
          Delay_ms (600);
          return true;
        }
        return true;
      }
    }
  }
  return false;
}

// Power Modeの設定 (CONFIGモードへの移行とモードの回復あり)
bool imu_set_powermode (TPowerMode p) {
  TOperationMode prevmode;
  TPowerMode prevpm;
  imu_read_byte (0, BNO055_P0ADDR_PWR_MODE, &prevpm);
  if (prevpm == p) return true;
  if (imu_set_opmode (opmode_CONFIG, &prevmode)) {
    if (imu_write_byte (0, BNO055_P0ADDR_PWR_MODE, p)) {
      return imu_set_opmode (prevmode, NULL);
    }
  }
  return false;
}

// 外部クロック選択 (CONFIGモードへの移行とモードの回復あり)
bool imu_set_ext_osc (bool ext) {
  TOperationMode prevmode;
  uint8_t tmp;
  if (imu_set_opmode (opmode_CONFIG, &prevmode)) {
    if (imu_read_byte (0, BNO055_P0ADDR_SYS_TRIGGER, &tmp)) {
      if (imu_write_byte (0, BNO055_P0ADDR_SYS_TRIGGER, (tmp & 0x7f) | (ext ? 0x80 : 0x00))) {
        return imu_set_opmode (prevmode, NULL);
      }
    }
  }
  return false;
}

// センサ配置選択 (CONFIGモードへの移行とモードの回復あり)
bool imu_set_axix_place (TAxisPlace p) {
  TOperationMode prevmode;
  if (imu_set_opmode (opmode_CONFIG, &prevmode)) {
    if (!imu_write_byte (0, BNO055_P0ADDR_AXIS_MAP_CONFIG, axisplace_conf_sign[p][0])) return false;
    if (!imu_write_byte (0, BNO055_P0ADDR_AXIS_MAP_SIGN, axisplace_conf_sign[p][1])) return false;
    return imu_set_opmode (prevmode, NULL);
  }
  return false;
}

// セルフテストおよび結果・システムステータス・システムエラー取得 (CONFIGモードへの移行とモードの回復あり)
/*
  - セルフテスト結果 (0:fail, 1:pass)
   bit 0 = 加速度センサ
   bit 1 = 地磁気センサ
   bit 2 = ジャイロセンサ
   bit 3 = マイクロコントローラ
  - システムステータス
   0 = Idle
   1 = System Error
   2 = Initializing Peripherals
   3 = System Initialization
   4 = Executing selftest
   5 = Sensor fusion algorithm running
   6 = System running without fusion algorithms
  - システムエラー
   0 = No error
   1 = Peripheral initialization error
   2 = System initialization error
   3 = Self test result failed
   4 = Register map value out of range
   5 = Register map address out of range
   6 = Register map write error
   7 = BNO low power mode not available for selected operation mode
   8 = Accelerometer power mode not available
   9 = Fusion algorithm configuration error
   10 = Sensor configuration error
*/
bool imu_get_tst_stat_err (uint8_t *tst, uint8_t *stat, uint8_t *err) {
  if (tst != NULL)  {
    uint8_t sys_trigger;
    TOperationMode prevmode;
    if (!imu_set_opmode (opmode_CONFIG, &prevmode)) return false;
    // Perform a self test.
    if (!imu_read_byte (0, BNO055_P0ADDR_SYS_TRIGGER, &sys_trigger)) return false;
    if (!imu_write_byte (0, BNO055_P0ADDR_SYS_TRIGGER, sys_trigger | 0x1)) return false;
    // Wait for self test to finish.
    Delay_ms (1000);
    if (!imu_read_byte (0, BNO055_P0ADDR_ST_RESULT, tst)) return false;
    if (!imu_set_opmode (prevmode, NULL)) return false;
  }
  if (stat != NULL) if (!imu_read_byte (0, BNO055_P0ADDR_SYS_STAT, stat)) return false;
  if (err != NULL) if (!imu_read_byte (0, BNO055_P0ADDR_SYS_ERR, err)) return false;
  return true;
}

// 初期化
bool init_imu (TOperationMode mode, TAxisPlace p) {
  uint8_t tmp;
  if (!imu_cehckid()) return false;

  // リセット(nRESET端子ではなくI2C経由)
  if (!imu_read_byte (0, BNO055_P0ADDR_SYS_TRIGGER, &tmp)) return false;
  if (!imu_write_byte (0, BNO055_P0ADDR_SYS_TRIGGER, tmp | 0x20)) return false;
  Delay_ms (650); // ココの遅延はトライアルアンドエラーで
  if (!imu_cehckid()) return false;

  // CONFIGモードに切り替え
  if (!imu_set_opmode (opmode_CONFIG, NULL)) return false;

  // 加速度センサのレンジを16Gに
  if (!imu_read_byte (1, BNO055_P1ADDR_ACC_CONFIG, &tmp)) return false;
  if (!imu_write_byte (1, BNO055_P1ADDR_ACC_CONFIG, (tmp & 0xfc) | 0x3)) return false;

  // ノーマルパワーモードに切り替え
  if (!imu_set_powermode (powmode_NORMAL)) return false;
  // 外部クロック選択
  if (!imu_set_ext_osc (true)) return false;

  // センサ配置設定
  if (!imu_set_axix_place (p)) return false;

  // 指定動作モードに切り替えて終了
  if (!imu_set_opmode (mode, NULL)) return false;
  return true;
}

#ifdef __cplusplus
}
#endif

#endif
