/*
  B3Mプロトコル ホスト

  半二重のRS-485 I/Fを介してB3Mシリーズと通信を行う。

  ・DYNAMIXELが1台も見つからなかったらプログラムを終了
  ・低レベルの関数ではなくDYNAMIXELのモデルを意識せずに物理値で指令等を行
    う関数を使用
  ・ボード上のTTL及びRS-485の通信ポートがDYNAMIXELとの通信で占有されてし
    まうのでprintfデバッグは不可能
  ・角度指令とLEDの明滅をFreeRTOSで個別のタスクで処理
  ・USARTライブラリの都合により1Mbpsを超えるボーレートは1000000,2000000,
    4000000bpsのみに対応
 */

#include  <FreeRTOS.h>
#include  <task.h>

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>
#include  <wl_delay.h>

// B3Mプロトコル処理関数
#include  <b3mlib.h>
#include  <b3mmemmap.h>

// シリアル通信用バッファ
uint8_t txb[100], rxb[100];

// B3MLIBデバイスID
TDeviceID devid;

#define TORQUE_LOCK   (0x03)
#define TORQUE_FREE   (0x02)
#define TORQUE_NORMAL (0x00)

#define TARGET_ID     (1)

#define DIF           (10)

// task 1
//-----------------------------
void T1 (void *pvParameters) {
  portTickType t = xTaskGetTickCount();
  int16_t ang, d = DIF;

  B3M_ReadWordData (devid, TARGET_ID, ADDRESS_CURRENT_POSITION, (uint16_t *)&ang, NULL);
  for (;;) {
      ang = ang + d;
      if (ang > 90) {
        ang = 90;
        d = -DIF;
      }
      if (ang < -90) {
        ang = -90;
        d = DIF;
      }
    // 角度を指令
    B3M_WriteWordData (devid, TARGET_ID, ADDRESS_DESIRED_POSITION, (uint16_t)ang, NULL);

    vTaskDelayUntil (&t, 5);
  }
}

// task 2
//-----------------------------
void T2 (void *pvParameters) {
  portTickType t = xTaskGetTickCount();

  while (1) {
    // LED2を反転
    Chip_GPIO_SetPinToggle (LPC_GPIO_PORT, 0, _LED2);
    vTaskDelayUntil (&t, 5);
  }
}

// usart
//-----------------------------
// usart初期化
uint32_t us_init (uint32_t baud) {
  return usart0_dma_init (Chip_Clock_GetFRGClockRate (0), baud, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
}

// ボーレート更新
uint32_t us_setbaud (uint32_t baud) {
  return usart0_dma_setbaudrate (Chip_Clock_GetFRGClockRate (0), baud);
}


// main
//-----------------------------
void main (void) {

  // B3MLIBパラメータ
  const TB3MHost_ConfParam conf = {
    us_init, us_setbaud, usart0_dma_rx_buff, usart0_dma_rx_purge, usart0_dma_putsbd, usart0_dma_getc
  };

  Delay_ms (1000);

  // B3MLIBオープン
  devid = B3M_OpenPort (&conf, 115200);

  // タイムアウトを10msに
  B3M_SetTimeOutOffset (devid, 10);

  // 全Dynamixelをスキャン
  if (B3M_Ping (devid, TARGET_ID, NULL)) {
    // トルクon
    if (B3M_WriteWordData (devid, TARGET_ID, ADDRESS_TORQUE_ON, TORQUE_NORMAL, NULL)) {
      // タスク作成
      xTaskCreate (T1, NULL, 300, NULL, 1, NULL);
      xTaskCreate (T2, NULL, 300, NULL, 1, NULL);
      // タスクスタート
      vTaskStartScheduler();
    }
  }
}
