﻿/*
  DXクライアント

  DYNAMIXELシリーズと同じネットワークに接続してクライアントとして機能さ
  せる。

  ・FreeRTOSが必要
  ・DX_PROTOCOL_VER マクロでV1及びV2のプロトコルを選択可能
  ・コントロールテーブルや各イベントに応じたコールバック関数を用意
  ・本ライブラリではコントロールテーブルのアイテムがModelInfomation～
    WriteNVMまでが必ず存在している事とする
  ・任意のアイテムを不揮発化できるが、アイテムの読み書きをしている間は揮
    発領域として扱い、WriteNVMを1にした時だけその時点の値をFLASHへ保存し
    次回起動時の初期値としてその値を使用
  ・パケット処理ルーチン(DX?_ClientCommTask)はタスクとして登録必須
  ・コールバック関数内ではコントロールテーブルへのアクセスに排他制御は不
    要
  ・任意のユーザータスクからコントロールテーブルへのアクセスする際は排他
    制御が必須(DX(2)_ReadMemやDX(2)_WriteMem)
  ・プロトコルV1の最大通信速度は2Mbps、V2は4Mbps(そのうち3Mbpsは不可)
  ・ウォッチドックタイマを活性化
*/

// DXプロトコルバージョン選択 (1 or 2)
#define DX_PROTOCOL_VER 1

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>
// DXクライアントライブラリ
#if DX_PROTOCOL_VER == 1
#include  <dxclient.h>
#elif DX_PROTOCOL_VER == 2
#include  <dx2client.h>
#endif
#include  <wl_delay.h>

#include  <FreeRTOS.h>
#include  <task.h>

// マクロと変数
//-----------------------------
// コントロールテーブル (定義必須)
typedef struct {
  uint16_t  ModelNumber;
#if DX_PROTOCOL_VER == 2
  uint32_t  ModelInfomation;
#endif
  uint8_t   FirmwareVersion;
  uint8_t   ID;
  uint8_t   Baudrate;
  uint8_t   WriteNVM;
  // ここより後は自由
  uint8_t   LED;
  uint8_t   Terminator;
  uint8_t   dummy[10];
} __attribute__ ((packed)) TCtrlTable;

TCtrlTable  *pmemmap;

// コントロールテーブルデフォルト値 (定義必須)
static const TCtrlTable DEFAULT_DATA = {
  ModelNumber     : 0,
#if DX_PROTOCOL_VER == 2
  ModelInfomation : 0,
#endif
  FirmwareVersion : 0,
  ID              : 1,
#if DX_PROTOCOL_VER == 1
  Baudrate        : 34,
#elif DX_PROTOCOL_VER == 2
  Baudrate        : 1,
#endif
  WriteNVM        : 0,
  LED             : 0,
  Terminator      : 1,
};

// シリアル通信バッファ
static uint8_t txd[200], rxd[200];

// USART
//-----------------------------
#if DX_PROTOCOL_VER == 2
// 使える使えないは別としてプロトコルとして対応するボーレート
static const uint32_t baud[9] = { 9600, 57600, 115200, 1000000, 2000000, 3000000, 4000000, 4500000, 10500000 };
#endif

// 初期化
#if DX_PROTOCOL_VER == 1
static uint32_t us_init (uint32_t baudrate) {
  return usart0_dma_init (Chip_Clock_GetFRGClockRate (0), baudrate, USIP_8N1, txd, sizeof (txd), rxd, sizeof (rxd));
}
#elif DX_PROTOCOL_VER == 2
static uint32_t us_init (uint8_t ind) {
  // USARTライブラリの都合で対応するインデックスを制限
  if (ind > 6) ind = 6;
  else if (ind == 5) ind = 4;
  return usart0_dma_init (Chip_Clock_GetFRGClockRate (0), baud[ind], USIP_8N1, txd, sizeof (txd), rxd, sizeof (rxd));
}
#endif

// ボーレート更新
#if DX_PROTOCOL_VER == 1
static uint32_t us_setbaudrate (uint32_t baudrate) {
  return usart0_dma_setbaudrate (Chip_Clock_GetFRGClockRate (0), baudrate);
}
#elif DX_PROTOCOL_VER == 2
static uint32_t us_setbaudrate (uint8_t ind) {
  // USARTライブラリの都合で対応するインデックスを制限
  if (ind > 6) ind = 6;
  else if (ind == 5) ind = 4;
  return usart0_dma_setbaudrate (Chip_Clock_GetFRGClockRate (0), baud[ind]);
}
#endif

// 終了
static void us_deinit (void) { }

// DXクライアントコールバック
//-----------------------------
// アイテムのレンジチェック (定義必須)
// 1アイテムへの書き込み操作時における値の範囲チェック
#if DX_PROTOCOL_VER == 1
static bool DX_CallbackValidPrevMapRange (const void *pmem, uint8_t u_st, uint8_t u_len) {
#elif DX_PROTOCOL_VER == 2
static bool DX_CallbackValidPrevMapRange (const void *pmem, uint16_t u_st, uint16_t u_len) {
#endif
  bool result = true;
#if DX_PROTOCOL_VER == 2
  const TCtrlTable *pNewControlTable = pmem;
#endif

#if DX_PROTOCOL_VER == 2
  if (IS_SAFE_VAR (pNewControlTable, pNewControlTable->Baudrate, u_st, u_len)) {
    if (! ((pNewControlTable->Baudrate <= 4) || (pNewControlTable->Baudrate == 6))) result = false;
  }
#endif

  return result;
}

// アイテムのレンジチェック (定義必須)
// 複数アイテムにまたがる書き込み操作時における各アイテムの値の範囲チェック
bool DX_CallbackValidNextMapRange (const void *pmem) {
#if DX_PROTOCOL_VER == 2
  const TCtrlTable *pNewControlTable = pmem;
#endif

  return (
#if DX_PROTOCOL_VER == 2
     ! ((pNewControlTable->Baudrate <= 4) || (pNewControlTable->Baudrate == 6))
     ||
#endif
     false
   );
}

// NVM初期化時
// 起動ないし再起動時にFLASHから書き戻したり機能の初期化が必要な場合
void DX_CallbackInitNVMValue (void *pCtrl, const void *pNVM) {
  TCtrlTable *pControlTable = pCtrl;
  const TCtrlTable *pNVMdata = pNVM;

  pControlTable->Terminator = pNVMdata->Terminator;
  Chip_GPIO_SetPinState (LPC_GPIO_PORT, 0, _TERM, (bool)pControlTable->Terminator);
}

// 書込パケット時
// アイテムへの書き込み操作時に更新があった場合の処理
void DX_CallbackWriteAction (const void *pNewCtrl, void *pCtrl, bool force) {
  const TCtrlTable *pNewControlTable = pNewCtrl;
  TCtrlTable *pControlTable = pCtrl;

  // LED On/Off
  if (pNewControlTable->LED != pControlTable->LED || force) {
    SetLED (pNewControlTable->LED);
  }
  // Terminator On/Off
  if (pNewControlTable->Terminator != pControlTable->Terminator || force) {
    Chip_GPIO_SetPinState (LPC_GPIO_PORT, 0, _TERM, (bool)pNewControlTable->Terminator);
  }
}

// TASK
//-----------------------------
// 通信タスク (定義必須)
void TaskDXComm (void *pvParameters) {
#if DX_PROTOCOL_VER == 1
  DX_ClientCommTask();
#elif DX_PROTOCOL_VER == 2
  DX2_ClientCommTask();
#endif
}

// main
//-----------------------------
void wdt_feed (void) { }

void sys_reset (void) {
  NVIC_SystemReset();
}

// クライアント機能初期化パラメータ (必須)
#if DX_PROTOCOL_VER == 1
static TDXClient_ConfParam ConfParam = {
  // コントロールテーブル等のシステム関連
  { (void *) &DEFAULT_DATA, sizeof (TCtrlTable), wdt_feed, sys_reset, 0xF800, 256 },
  // USART関連
  { us_init, us_deinit, us_setbaudrate, usart0_dma_tx_buff, usart0_dma_rx_buff, usart0_dma_tx_purge, usart0_dma_rx_purge, usart0_dma_putsbd, usart0_dma_getc },
  // コールバック関数
  { DX_CallbackValidPrevMapRange, DX_CallbackValidNextMapRange, DX_CallbackInitNVMValue, NULL, DX_CallbackWriteAction }
};
#elif DX_PROTOCOL_VER == 2
static TDX2Client_ConfParam ConfParam = {
  { (void *) &DEFAULT_DATA, sizeof (TCtrlTable), true, NULL, 0, NULL, wdt_feed, sys_reset, 0xF800, 256 },
  { us_init, us_deinit, us_setbaudrate, usart0_dma_tx_buff, usart0_dma_rx_buff, usart0_dma_tx_purge, usart0_dma_rx_purge, usart0_dma_putsbd, usart0_dma_getc },
  { DX_CallbackValidPrevMapRange, DX_CallbackValidNextMapRange, DX_CallbackInitNVMValue, NULL, DX_CallbackWriteAction },
};
#endif

void main (void) {
  // DXクライアント機能初期化 (必須)
#if DX_PROTOCOL_VER == 1
  if (DX_Config (&ConfParam)) {
    pmemmap = DX_GetPCtrlTalbe();
#elif DX_PROTOCOL_VER == 2
  if (DX2_Config (&ConfParam)) {
    pmemmap = DX2_GetPCtrlTalbe();
#endif
    // 通信タスク登録 (必須)
    xTaskCreate (TaskDXComm, NULL, 300, NULL, 1, NULL);
    vTaskStartScheduler();
  }
}
