﻿/*
  USART

  半二重のRS-485ないしTTL I/Fによるシリアル通信。
  USART用のライブラリはポーリング・割り込み・DMAの3種類用意されている。
  いずれも使用する側のコード上の表面的な差は無いが、ポーリング版は一生懸
  命USARTのレジスタを監視しながら通信処理を行い、割り込み版はその名の通
  りレジスタの変化を割り込みで捉えながら通信処理を行い、DMA版は送受信デー
  タのやりとりをDMAを介して行うといった違いがある。

  ・端子の初期化は e097b_def.h を参照の事
  ・ボード上のRS-485とTTL I/Fは区別されず、送信は両者同時、受信は論理和
    で処理
  ・半二重なのはトランシーバ部分だけであり、それ以外は実質全二重
  ・1バイト毎に返信しているが、物理層が半二重にもかかわらずどこもかしこ
    もそれを全く意識していないので、連続的にホストから送信があった場合に
    コンフリクトが生じまくってターミナル上ではまともな表示が得られなくな
    る
  ・USARTの送信開始と共にRTSが自動的にHIGHになりトランシーバーの送信がイ
    ネーブルされ、データを完全に送信し終えるとRTSがLOWになりトランシー
    バーの受信がイネーブルになる事で半二重化
  ・USARTへの供給クロックは e097_def.h の初期化部で32MHzを供給する設定を
    行っているので、最大ボーレートはそれを8で割った4Mbpsとなる
  ・USART3～4にはRTS端子が何故か無いので、本ボードの半二重I/F制御用には
    USART0～2のみが使用できる
  ・RS-485トランシーバに内蔵されたターミネータはプログラムからON/OFFでき
    るが、PIO_Configureによる端子の初期化時にONになっている
  ・TTL I/Fのみを使用する際は使用しないRS-485に装備されたターミネータを
    必ずONにし受信回路のハイインピーダンス状態を回避する事
  ・主にポーリング版は時に割り込みルーチン内で使え、割り込み版は省電力
    モードからの復帰に使え、DMA版は送受データとUSART間のやりとり時にCPU
    の能力をほとんど使わないのでその間に他の処理が行えるといった使い分け
    ができる
  ・書式化文字列関数(scanf,printf)は簡易版のため実数に対応しない
  ・いずれも初期化時に2バイト以上の送受信バッファを必要とする
  ・ポーリング版のバッファは他との体裁を合わせるためだけに引数を受け取る
    ので、広大なバッファを用意する意味は皆無。
  ・割り込み及びDMA版の受信バッファサイズは大きいほどバッファからデータ
    を取り出すタイミングに余裕ができる
*/

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
// USART用関数
#include  <us.h>
#include  <wl_delay.h>

// 処理方法による関数の選択 (1:ポーリング, 2:割り込み, 3:DMA)
#define USART_TYPE 1

// ポーリング版
#if USART_TYPE == 1
#define us_init     usart0_init             // 初期化
#define us_setbaudrate usart0_setbaudrate   // ボーレート変更
#define us_tx_buff  usart0_tx_buff          // 残り送信バイト数
#define us_rx_buff  usart0_rx_buff          // 受信データの未取り出しバイト数
#define us_tx_purge usart0_tx_purge         // 送信データ一掃
#define us_rx_purge usart0_rx_purge         // 受信データ一掃
#define us_putc     usart0_putc             // 1バイト送信
#define us_puts     usart0_puts             // 文字列送信
#define us_putsb    usart0_putsb            // バイト列送信
#define us_getc     usart0_getc             // 1バイト受信待ち
#define us_gets     usart0_gets             // 文字列受信待ち
#define us_printf   usart0_printf           // 書式化文字列送信
#define us_scanf    usart0_scanf            // 書式化文字列受信
// 割り込み版
#elif USART_TYPE == 2
#define us_init     usart0_int_init
#define us_setbaudrate usart0_int_setbaudrate
#define us_tx_buff  usart0_int_tx_buff
#define us_rx_buff  usart0_int_rx_buff
#define us_tx_purge usart0_int_tx_purge
#define us_rx_purge usart0_int_rx_purge
#define us_putc     usart0_int_putc
#define us_puts     usart0_int_puts
#define us_putsb    usart0_int_putsb
#define us_getc     usart0_int_getc
#define us_gets     usart0_int_gets
#define us_printf   usart0_int_printf
#define us_scanf    usart0_int_scanf
// DMA版
#elif USART_TYPE == 3
#define us_init     usart0_dma_init
#define us_setbaudrate usart0_dma_setbaudrate
#define us_tx_buff  usart0_dma_tx_buff
#define us_rx_buff  usart0_dma_rx_buff
#define us_tx_purge usart0_dma_tx_purge
#define us_rx_purge usart0_dma_rx_purge
#define us_putc     usart0_dma_putc
#define us_puts     usart0_dma_puts
#define us_putsb    usart0_dma_putsb
#define us_getc     usart0_dma_getc
#define us_gets     usart0_dma_gets
#define us_printf   usart0_dma_printf
#define us_scanf    usart0_dma_scanf
#endif

// main
//-----------------------------
void main (void) {
  // シリアル通信バッファ
  uint8_t txd[2], rxd[2];

  // USART0を初期化
  // USART0のクロックソースはfrg0_src_clkを元にしたFRG0を選択済みなので、
  // Chip_Clock_GetFRGClockRate (0)は32,000,000Hzを返す
  us_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txd, sizeof (txd), rxd, sizeof (rxd));

  // 文字列送信
  us_puts ("\nHELLO!\n");

  char s[100], c;
  int i = 0;
  for (;;) {
    // 1文字受信 (受信データが得られるまで待機)
    c = us_getc();
    // 1文字送信 (受信したデータをそのまま返信)
    us_putc (c);
    // 書式化文字列送信
    us_printf ("(%d)", i);
    switch (c) {
      case '0':
        us_puts ("\n>Input string = ");
        // 書式化文字列受信
        us_scanf ("%s", &s);
        us_printf ("\n string = %s\n", s);
        break;
      case '1':
        us_puts ("\n>Input number = ");
        us_scanf ("%d", &i);
        us_printf ("\n number = %d\n", i);
        break;
      case '2':
        us_puts ("\n>Term ON\n");
        SetTerminator (true);
        break;
      case '3':
        us_puts ("\n>Term OFF\n");
        SetTerminator (false);
        break;
      case '!':
        us_puts ("\n>BYE!\n ");
        Delay_ms (50);
        NVIC_SystemReset();
        break;
    }
    i++;
  }
}
