﻿/*
  SysTick Time & Sleep

  ARMのシステムタイマを使い1kHz周期で割り込みを発生させるよう設定。
  割り込み内でカウンタをインクリメントし、1000で割り切れるタイミングで
  LEDの状態を反転。
  メインのループ内で常にSLEEPモードへの移行を励起。

  ・FROの精度が±1%なので時計としてはまず使えず、分オーダになれば積算誤
    差は歴然としてくる
  ・CMSISで提供される関数を使用してシステムタイマを初期化
  ・システムタイマはクロックと同期してカウントアップし、SysTick_Configの
    引数は-1されて再ロードレジスタに渡される
  ・再ロードレジスタは最大24ビットなので、SysTick_Configはそれを超えた値
    が指定されると失敗する
  ・SysTick_Configは成功すると0、失敗すると1を返す
  ・WFI命令で割り込み発生までSLEEPモードに遷移させている
*/
#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>

// マクロ・グローバル変数など
//-----------------------------
// カウンタ
uint32_t tickcnt;

// Interrupt handler
//-----------------------------
void SysTick_Handler (void) {
  tickcnt++;

  // 1000回に1回のタイミングでLEDの明滅を反転
  if (tickcnt % 1000 == 0) ToggleLED();
}

// main
//-----------------------------
void main (void) {
  uint8_t txd[20], rxd[20];

  // ポーリング版ライブラリでUSART0を使用
  usart0_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txd, sizeof (txd), rxd, sizeof (rxd));

  // システムタイマの設定 (カウンタの初期化と再ロードレジスタの設定、割り込み許可)
  // ここでは30000000[Hz]/1000[Hz]=30000を渡す事で1000Hz周期で割り込みが発生
  if (SysTick_Config (F_CPU / 1000) == 0) {
    for (;;) {
      // スリープモード移行
      __WFI();
      // タイマ割り込みが発生すると復帰して以下に進む

      if (usart0_rx_buff()) {
        switch (usart0_getc()) {
          case '!':
            NVIC_SystemReset();
            break;
          default:
            usart0_printf ("\rtickcnt=%lu        ", tickcnt);
            break;
        }
      }
    }
  }
}
