﻿/*
  GPIO・ADC・DAC

  外部端子(CN1)をデジタル入出力及びアナログ入出力に。
  端子機能の設定ライブラリで初期化し、デジタル出力時は出力信号を変化さ
  せ、デジタル入力時はその値を取得、アナログ入力時はA/D変換値を取得、ア
  ナログ出力時はDACで電圧出力を行う。

  ・プログラム上のビットアサインとCN1との対比はADCの追番に準ずる
  ・デジタル出力の場合はプッシュプル構成
  ・デジタル/アナログ入力の場合はハイインピーダンス構成
  ・GPIOの端子状態取得や出力状態の変更はLPCOpenの関数を使用、アナログ入
    力はADCライブラリを使用
  ・ADCの分解能は12ビット
  ・ADCのクロックソースは個別に設定する必要がある
  ・ADC_Initのクロックレートはシステムクロックを指定クロックレートで割っ
    た値が使用されるが、レジスタが8ビット幅なので30MHz/256=117,187Hz最
    小値、最大値は1MHz
  ・DACは1チャンネルのみ、分解能は10ビット
  ・DACは端子機能設定ライブラリの初期化の他にIOCONで追加の設定が必要
  ・リセット端子の機能を無効にし入力端子に構成
*/

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>
#include  <wl_adc.h>
#include  <wl_delay.h>

// マクロ・グローバル変数など
//-----------------------------
// CN1の端子全てを出力
static const TPin pins_out[] = {
  { _GPIO0,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO1,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO2,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO3,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO4,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO5,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO6,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO7,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO8,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO9,  PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO10, PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
  { _GPIO11, PIO_TYPE_OUTPUT_0, 0,                PIO_MODE_DEFAULT },
};

// CN1の端子全てを入力
static const TPin pins_in[] = {
  { _GPIO0,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO1,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO2,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO3,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO4,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO5,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO6,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO7,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO8,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO9,  PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO10, PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },
  { _GPIO11, PIO_TYPE_INPUT,    0,                PIO_MODE_DEFAULT },

  { _PB1,    PIO_TYPE_INPUT,    0,                PIO_MODE_PULLUP  },
};

// CN1の端子全てをアナログ入力
static const TPin pins_adc[] = {
  { _GPIO0,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_0,  PIO_MODE_DEFAULT },
  { _GPIO1,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_1,  PIO_MODE_DEFAULT },
  { _GPIO2,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_2,  PIO_MODE_DEFAULT },
  { _GPIO3,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_3,  PIO_MODE_DEFAULT },
  { _GPIO4,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_4,  PIO_MODE_DEFAULT },
  { _GPIO5,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_5,  PIO_MODE_DEFAULT },
  { _GPIO6,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_6,  PIO_MODE_DEFAULT },
  { _GPIO7,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_7,  PIO_MODE_DEFAULT },
  { _GPIO8,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_8,  PIO_MODE_DEFAULT },
  { _GPIO9,  PIO_TYPE_FIXED,    SWM_FIXED_ADC_9,  PIO_MODE_DEFAULT },
  { _GPIO10, PIO_TYPE_FIXED,    SWM_FIXED_ADC_10, PIO_MODE_DEFAULT },
  { _GPIO11, PIO_TYPE_FIXED,    SWM_FIXED_ADC_11, PIO_MODE_DEFAULT },
};

// CN1-10のみアナログ出力
static const TPin pins_dac[] = {
  { _GPIO9,  PIO_TYPE_FIXED,    SWM_FIXED_DAC_0,  PIO_MODE_DEFAULT },
};

// PB
//-----------------------------
bool GetPB (void) {
  return !Chip_GPIO_GetPinState (LPC_GPIO_PORT, 0, _PB1);
}

// GPIO
//-----------------------------
// MPUのGPIOとプログラム上のビットアサインの変換用
static const uint8_t GPIO_PinNo[12] = { 7, 6, 14, 23, 22, 21, 20, 19, 18, 17, 13, 4 };

// デジタル入力値取得
uint16_t gpio_in (void) {
  uint16_t result = 0;
  for (int i = 0; i < 12; i++) result |= (Chip_GPIO_GetPinState (LPC_GPIO_PORT, 0, GPIO_PinNo[i]) ? (1 << i) : 0);
  return result;
}

// 任意値のデジタル出力
void gpio_out (uint16_t outdata) {
  for (int i = 0; i < 12; i++) Chip_GPIO_SetPinState (LPC_GPIO_PORT, 0, GPIO_PinNo[i], outdata & (1 << i) ? true : false);
}

// main
//-----------------------------
void main (void) {
  char c;
  uint16_t data = 0;
  uint8_t txd[100], rxd[10];

  usart0_dma_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txd, sizeof (txd), rxd, sizeof (rxd));
  usart0_dma_puts ("\nSelect [0] Digital Input, [1] Digital Output, [2] Analog Input [3] Analog Output ->");
  usart0_dma_putc (c = usart0_dma_getc());
  switch (c) {
    case '0': // デジタル入力
      usart0_dma_printf ("\nConfigure GPIO as input.\n");
      PIO_Configure (pins_in, PIO_LISTSIZE (pins_in));
      while (1) {
        ToggleLED();
        usart0_dma_printf ("\rin data=$%03X pb=%d", gpio_in(), GetPB());
        if (usart0_dma_rx_buff()) if (usart0_dma_getc() == '!') NVIC_SystemReset();
        Delay_ms (100);
      }
      break;
    case '1': // デジタル出力
      usart0_dma_printf ("\nConfigure GPIO as output.\n");
      PIO_Configure (pins_out, PIO_LISTSIZE (pins_out));
      while (1) {
        ToggleLED();
        usart0_dma_printf ("\rout data=$%03X ", data);
        gpio_out (data);
        if (++data > 0xfff) data = 0;
        if (usart0_dma_rx_buff()) if (usart0_dma_getc() == '!') NVIC_SystemReset();
        Delay_ms (10);
      }
      break;
    case '2': // アナログ入力
      usart0_dma_printf ("\nConfigure GPIO as ADC.\n");
      PIO_Configure (pins_adc, PIO_LISTSIZE (pins_adc));
      // ADCのクロック源を選択 (ADCライブラリでは触れていない)
      LPC_SYSCON->ADCCLKSEL = 0; // Use fro_clk as source for ADC async clock (fro)
      LPC_SYSCON->ADCCLKDIV = 1; // Enable clock, and divide-by-1 at this clock divider
      // ADC初期化 (200kHz 全12チャンネル活性化 バーストモード)
      ADC_Init (200000, 0xfff);
      while (1) {
        ToggleLED();
        usart0_dma_puts ("\r");
        // 全チャンネルの変換結果取得
        for (int i = 0; i < 12; i++) usart0_dma_printf ("%4d ", ADC_Get (i));
        if (usart0_dma_rx_buff()) if (usart0_dma_getc() == '!') NVIC_SystemReset();
        Delay_ms (100);
      }
      break;
    case '3': // アナログ出力
      usart0_dma_printf ("\nConfigure GPIO9 as DAC.\n");
      PIO_Configure (pins_dac, PIO_LISTSIZE (pins_dac));
      LPC_IOCON->PIO0_17 |= (1 << IOCON_PIO_DACMODE_P);
      // DACのクロックを活性化
      Chip_DAC0_Init();
      // 割り込み及びDMAを不使用
      Chip_DAC_ConfigDMAConverterControl (LPC_DAC0, 0);
      Chip_DAC_SetDMATimeOut (LPC_DAC0, 0);
      while (1) {
        ToggleLED();
        for (int i = 0; i < 1024; i++) {
          usart0_dma_printf ("\rdac=%4d ", i);
          Chip_DAC_UpdateValue (LPC_DAC0, i);
          if (usart0_dma_rx_buff()) if (usart0_dma_getc() == '!') NVIC_SystemReset();
          Delay_ms (5);
        }
      }
      break;
  }
}
