/*
  FreeRTOS

  FreeRTOSによるマルチタスク。

  ・FreeRTOSConfig.hに設定した上でcortex-m0用にコンパイルしたFreeRTOS
    をライブラリとして提供
  ・全てのタスクがBlocked状態になると積極的に省電力モードに遷移させてい
    る
  ・システムリセットを切り離して入力端子にし、PB1を押している間はミュー
    テックスで他のタスクと排他制御する
  ・xTaskCreateのusStackDepthは意図的に小さく設定してある
 */

#include  <FreeRTOS.h>
#include  <task.h>
#include  <semphr.h>

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>
#include  <wl_delay.h>

// 端子設定
const TPin pins[] = {
  { _PB1, PIO_TYPE_INPUT, 0, PIO_MODE_PULLUP },
};

// シリアル通信用バッファ
uint8_t txb[1000], rxb[1000];

// mutex
SemaphoreHandle_t xMutex;

// PB
//-----------------------------
bool GetPB (void) {
  return !Chip_GPIO_GetPinState (LPC_GPIO_PORT, 0, _PB1);
}

// タスク
//-----------------------------
void T1 (void *pvParameters) {
  portTickType t = xTaskGetTickCount();

  for (;;) {
    xSemaphoreTake (xMutex, portMAX_DELAY);
    xSemaphoreGive (xMutex);

    ToggleLED();

    vTaskDelayUntil (&t, 200);
  }
}

void T2 (void *pvParameters) {
  portTickType t = xTaskGetTickCount();

  for (;;) {
    xSemaphoreTake (xMutex, portMAX_DELAY);
    xSemaphoreGive (xMutex);

    usart0_dma_puts ("*");

    vTaskDelayUntil (&t, 500);
  }
}

void T3 (void *pvParameters) {
  int i, r;
  static uint8_t rbuf[100];

  for (;;) {
    if ((r = usart0_dma_rx_buff())) {
      for (i = 0; i < r && i < sizeof (rbuf); i++) {
        rbuf[i] = usart0_dma_getc();
        if (rbuf[i] == '!') NVIC_SystemReset();
      }
      usart0_dma_putsb (rbuf, i);
    }
    vTaskDelay (100);
  }
}

void T4 (void *pvParameters) {
  for (;;) {
    xSemaphoreTake (xMutex, portMAX_DELAY);
    while (GetPB()) ;
    xSemaphoreGive (xMutex);

    vTaskDelay (10);
  }
}

// Stack over flow hook
//-----------------------------------
// スタックオーバーフローを検出 (確実に補足できるわけでもなく)
void vApplicationStackOverflowHook (TaskHandle_t pxTask, char *pcTaskName) {
  taskDISABLE_INTERRUPTS();

  usart0_dma_printf ("\nstack overflow(%s)", pcTaskName);

  while (1) {
    // LEDの輝度を一定周期で変える
    for (int i = 0; i < 500; i++) {
      SetLED (true);
      Delay_us (500 - i);
      SetLED (false);
      Delay_us (i);
    }
    Delay_ms (50);
  }
}

// main
//-----------------------------
void main (void) {
  // PB1の端子機能の初期化
  PIO_Configure (pins, PIO_LISTSIZE (pins));

  usart0_dma_init (Chip_Clock_GetFRGClockRate (0), 115200, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));

  // ミューテックス作成
  xMutex = xSemaphoreCreateMutex();

  // タスク作成
  xTaskCreate (T1, "T1", 40, NULL, 1, NULL);
  xTaskCreate (T2, "T2", 40, NULL, 1, NULL);
  xTaskCreate (T3, "T3", 40, NULL, 1, NULL);
  xTaskCreate (T4, "T4", 40, NULL, 1, NULL);
  // OSの動作開始
  vTaskStartScheduler();
}
