/*
  DYNMIXELプロトコル ホスト basic

  半二重のRS-485ないしTTL I/Fを介してDYNAMIXELと通信を行う。

  ・マクロのDX_PROTOCOL_VERに使用するプロトコルのバージョンを設定
  ・プロトコルV1のDYNAMIXEL MXシリーズの場合は、通信速度は57600bps、対象
    IDが1の出荷時初期状態とする
  ・プロトコルV2のDYNAMIXEL Xシリーズの場合は、通信速度は57600bps、対象
    IDが1の出荷時初期状態とする
  ・コントロールテーブルを読み書きする低レベルな関数を用いてホーンを往復
    運動
  ・トルクイネーブルと現在位置の読み出しが成功しなかったら位置の指令を行
    わずにプログラムを終了
  ・ボード上のTTL及びRS-485の通信ポートがDYNAMIXELとの通信で占有されてし
    まうので、そのままではprintfデバッグは不可能
  ・DX(2)LIBはFreeRTOSありきの構成のため、コード上でFreeRTOS関連の宣言を
    使っていないとしてもFreeRTOSのライブラリがリンクされ、一部FreeRTOSの
    機能が生きている
  ・DYNAMIXEL MXないしXシリーズ以外ではコントロールテーブル上に割り当て
    られたアイテムのアドレスやレンジが異なる可能性があるので注意
 */

// DYNAMIXELプロトコルバージョン選択 (1 or 2)
#define DX_PROTOCOL_VER 1

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>
#include  <wl_delay.h>

// DYNAMIXELプロトコル処理関数
#if DX_PROTOCOL_VER == 1
#include  <dxlib.h>
#include  <dxmemmap.h>
#elif DX_PROTOCOL_VER == 2
#include  <dx2lib.h>
#include  <dx2memmap.h>
#endif

// マクロとグローバル変数
//-----------------------------

// 対処のDYNAMIXELのIDとボーレート
#define _TARGET_ID  1
#define _BAUDRATE   57600

// シリアル通信用バッファ
uint8_t txb[100], rxb[100];

// usart
//-----------------------------
uint32_t us_init (uint32_t baud) {
  return usart0_dma_init (Chip_Clock_GetFRGClockRate (0), baud, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
}

uint32_t us_setbaud (uint32_t baud) {
  return usart0_dma_setbaudrate (Chip_Clock_GetFRGClockRate (0), baud);
}

// main
//-----------------------------
void main (void) {
  // デバイスID (オープン時に返される値)
  TDeviceID devid;
  // エラーコード (DYNAMIXELから返るエラーとライブラリのエラー)
  TErrorCode err;

  // ボードとDYNAMIXELの電源が同時に開始された場合を考慮しDYNAMIXELが起動完了するまで少し待機
  Delay_ms (1000);

#if DX_PROTOCOL_VER == 1
  const TDXHost_ConfParam conf = {
    us_init,                // USARTの初期化関数
    us_setbaud,             // USARTのボーレート設定関数
    usart0_dma_rx_buff,     // USARTの受信バッファ取得関数
    usart0_dma_rx_purge,    // USARTの受信バッファ破棄関数
    usart0_dma_putsbd,      // USARTの指定バイト数送信関数
    usart0_dma_getc         // USARTの1バイト受信関数
  };

  // confで指定されたポートと指定ボーレートでDX2LIBをオープン
  devid = DX_OpenPort (&conf, _BAUDRATE);
  // タイムアウトのオフセットを5msに
  DX_SetTimeOutOffset (devid, 5);

  // トルクイネーブル (8ビット幅)
  if (DX_WriteByteData (devid, _TARGET_ID, ADDRESS_TORQUE_ENABLE, 1, &err)) {
    uint16_t pos;
    // 現在位置取得 (16ビット幅)
    if (DX_ReadWordData (devid, _TARGET_ID, ADDRESS_PRESENT_POSITION, &pos, &err)) {
      while (1) {
        // LED ON (8ビット幅)
        DX_WriteByteData (devid, _TARGET_ID, ADDRESS_LED, 1, &err);
        for (; pos < 3000; pos += 10) {
          // 3000を超えるまで位置を10ずつ増やして指令 (16ビット幅)
          DX_WriteWordData (devid, _TARGET_ID, ADDRESS_GOAL_POSITION, pos, &err);
          Delay_ms (10);
        }
        // LED OFF (8ビット幅)
        DX_WriteByteData (devid, _TARGET_ID, ADDRESS_LED, 0, &err);
        for (; pos > 1000; pos -= 10) {
          // 1000を下回るまで位置を10ずつ減らして指令 (16ビット幅)
          DX_WriteWordData (devid, _TARGET_ID, ADDRESS_GOAL_POSITION, pos, &err);
          Delay_ms (10);
        }
      }
    }
  }
#elif DX_PROTOCOL_VER == 2
  const TDX2Host_ConfParam conf = {
    true,                   // CRCをペリフェラルで算出するか否か
    us_init,                // USARTの初期化関数
    us_setbaud,             // USARTのボーレート設定関数
    usart0_dma_rx_buff,     // USARTの受信バッファ取得関数
    usart0_dma_rx_purge,    // USARTの受信バッファ破棄関数
    usart0_dma_putsbd,      // USARTの指定バイト数送信関数
    usart0_dma_getc         // USARTの1バイト受信関数
  };

  // confで指定されたポートと指定ボーレートでDX2LIBをオープン
  devid = DX2_OpenPort (&conf, _BAUDRATE);
  // タイムアウトのオフセットを5msに
  DX2_SetTimeOutOffset (devid, 5);

  // トルクイネーブル (8ビット幅)
  if (DX2_WriteByteData (devid, _TARGET_ID, ADDRESS_X_TORQUE_ENABLE, 1, &err)) {
    int32_t pos;
    // 現在位置取得 (16ビット幅)
    if (DX2_ReadLongData (devid, _TARGET_ID, ADDRESS_X_PRESENT_POSITION, (uint32_t *)&pos, &err)) {
      while (1) {
        // LED ON (8ビット幅)
        DX2_WriteByteData (devid, _TARGET_ID, ADDRESS_X_LED_RED, 1, &err);
        for (; pos < 3000; pos += 10) {
          // 3000を超えるまで位置を10ずつ増やして指令 (32ビット幅)
          DX2_WriteLongData (devid, _TARGET_ID, ADDRESS_X_GOAL_POSITION, pos, &err);
          Delay_ms (10);
        }
        // LED OFF (8ビット幅)
        DX2_WriteByteData (devid, _TARGET_ID, ADDRESS_X_LED_RED, 0, &err);
        for (; pos > 1000; pos -= 10) {
          // 1000を下回るまで位置を10ずつ減らして指令 (32ビット幅)
          DX2_WriteLongData (devid, _TARGET_ID, ADDRESS_X_GOAL_POSITION, pos, &err);
          Delay_ms (10);
        }
      }
    }
  }
#endif
}
