/*
  DYNMIXELプロトコル ホスト 追加API

  半二重のRS-485ないしTTL I/Fを介してDYNAMIXELと通信を行う。

  ・マクロのDX_PROTOCOL_VERに使用するプロトコルのバージョンを設定
  ・プロトコルV1のDYNAMIXEL MXシリーズの場合は、通信速度は57600bps、対象
    IDと数は問わない
  ・プロトコルV2のDYNAMIXEL Xシリーズの場合は、通信速度は57600bps、対象
    IDと数は問わない
  ・DYNAMIXELが1台も見つからなかったらプログラムを終了
  ・低レベルの関数ではなくDYNAMIXELのモデルを意識せずに物理値で指令等を行
    う関数を使用
  ・ボード上のTTL及びRS-485の通信ポートがDYNAMIXELとの通信で占有されてし
    まうのでprintfデバッグは不可能
  ・角度指令とLEDの明滅をFreeRTOSで個別のタスクで処理
  ・USARTライブラリの都合により1Mbpsを超えるボーレートは1000000,2000000,
    4000000bpsのみに対応
 */

// DYNAMIXELプロトコルバージョン選択 (1 or 2)
#define DX_PROTOCOL_VER 1

#include  <FreeRTOS.h>
#include  <task.h>

#include  <chip.h>
// ボードの端子割り当てと最小限の初期化
#include  "e097b_def.h"
#include  <us.h>
#include  <wl_delay.h>

// DYNAMIXELプロトコル処理関数
#if DX_PROTOCOL_VER == 1
#include  <dxlib.h>
#elif DX_PROTOCOL_VER == 2
#include  <dx2lib.h>
#endif

// マクロとグローバル変数
//-----------------------------
#if DX_PROTOCOL_VER == 1
#define DXL_OpenPort                  DX_OpenPort
#define DXL_SetTimeOutOffset          DX_SetTimeOutOffset
#define DXL_ScanDevices               DX_ScanDevices
#define DXL_SetTorqueEnablesEquival   DX_SetTorqueEnablesEquival
#define DXL_SetOperatingModesEquival  DX_SetOperatingModesEquival
#define DXL_GetPresentAngles          DX_GetPresentAngles
#define DXL_SetGoalAngles             DX_SetGoalAngles
#define DXL_SetLED                    DX_SetLED
#elif DX_PROTOCOL_VER == 2
#define DXL_OpenPort                  DX2_OpenPort
#define DXL_SetTimeOutOffset          DX2_SetTimeOutOffset
#define DXL_ScanDevices               DX2_ScanDevices
#define DXL_SetTorqueEnablesEquival   DX2_SetTorqueEnablesEquival
#define DXL_SetOperatingModesEquival  DX2_SetOperatingModesEquival
#define DXL_GetPresentAngles          DX2_GetPresentAngles
#define DXL_SetGoalAngles             DX2_SetGoalAngles
#define DXL_SetLED                    DX2_SetLED
#endif

// シリアル通信用バッファ
uint8_t txb[100], rxb[100];

// DX(2)LIBデバイスID
TDeviceID devid;

// 制御対象とするDYNAMIXELのID一覧
uint8_t ids[254];
// 検索で見つかったDYNAMIXELの数
int detectnum = 0;

// 角度の増分
#define DIF (0.3)

// task 1
//-----------------------------
void T1 (void *pvParameters) {
  portTickType t = xTaskGetTickCount();
  double ang[detectnum], d[detectnum];

  for (int i = 0; i < detectnum; i++) d[i] = DIF;
  // 現在の角度を取得し初期値とする
  DXL_GetPresentAngles (devid, ids, ang, detectnum);
  for (;;) {
    for (int i = 0; i < detectnum; i++) {
      ang[i] = ang[i] + d[i];
      if (ang[i] > 90) {
        ang[i] = 90;
        d[i] = -DIF;
      }
      if (ang[i] < -90) {
        ang[i] = -90;
        d[i] = DIF;
      }
    }
    // 角度を一括指令
    DXL_SetGoalAngles (devid, ids, ang, detectnum);
    vTaskDelayUntil (&t, 5);
  }
}

// task 2
//-----------------------------
void T2 (void *pvParameters) {
  bool b = false;
  portTickType t = xTaskGetTickCount();

  while (1) {
    // LED2を反転
    Chip_GPIO_SetPinToggle (LPC_GPIO_PORT, 0, _LED2);
    // DynamixelのLEDを設定
    for (int i = 0; i < detectnum; i++) DXL_SetLED (devid, ids[i], b);
    b = !b;
    vTaskDelayUntil (&t, 5);
  }
}

// usart
//-----------------------------
// usart初期化
uint32_t us_init (uint32_t baud) {
  return usart0_dma_init (Chip_Clock_GetFRGClockRate (0), baud, USIP_8N1, txb, sizeof (txb), rxb, sizeof (rxb));
}

// ボーレート更新
uint32_t us_setbaud (uint32_t baud) {
  return usart0_dma_setbaudrate (Chip_Clock_GetFRGClockRate (0), baud);
}

// main
//-----------------------------
void main (void) {

  // DX(2)LIBパラメータ
  #if DX_PROTOCOL_VER == 1
  const TDXHost_ConfParam conf = {
    us_init, us_setbaud, usart0_dma_rx_buff, usart0_dma_rx_purge, usart0_dma_putsbd, usart0_dma_getc
  };
  #elif DX_PROTOCOL_VER == 2
  const TDX2Host_ConfParam conf = {
    true,
    us_init, us_setbaud, usart0_dma_rx_buff, usart0_dma_rx_purge, usart0_dma_putsbd, usart0_dma_getc
  };
  #endif

  Delay_ms (1000);

  // DX(2)LIBオープン
  devid = DXL_OpenPort (&conf, 57600);
  // タイムアウトを2msに
  DXL_SetTimeOutOffset (devid, 2);

  // 全Dynamixelをスキャン
  if ((detectnum = DXL_ScanDevices (devid, ids))) {
    // トルクoff
    DXL_SetTorqueEnablesEquival (devid, ids, detectnum, false);
    // ジョイントモードに設定
    if (DXL_SetOperatingModesEquival (devid, ids, detectnum, 3)) {
      // トルクon
      if (DXL_SetTorqueEnablesEquival (devid, ids, detectnum, true)) {
        // タスク作成
        xTaskCreate (T1, NULL, 300, NULL, 1, NULL);
        xTaskCreate (T2, NULL, 300, NULL, 1, NULL);
        // タスクスタート
        vTaskStartScheduler();
      }
    }
  }
}
